@@ -302,7 +302,11 @@ class CiscoStyleCli:
             returnable = ""
             desc = ""
             argumentType = None
-            if '__attribute' in v:
+            prefunc = None
+            returnfunc=None
+            additionalDict=None
+            additionalList=None
+            if v and '__attribute' in v:
                 if 'type' in v['__attribute']:
                     type = v['__attribute']['type']
                 if 'returnable' in v['__attribute']:
@@ -311,11 +315,20 @@ class CiscoStyleCli:
                     desc = v['__attribute']['desc']
                 if 'argument-type' in v['__attribute']:
                     argumentType = v['__attribute']['argument-type']
+                if 'prefunc' in v['__attribute']:
+                    prefunc = v['__attribute']['prefunc']
+                if 'returnfunc' in v['__attribute']:
+                    returnfunc = v['__attribute']['returnfunc']
+                if 'additionalDict' in v['__attribute']:
+                    additionalDict = v['__attribute']['additionalDict']
+                if 'additionalList' in v['__attribute']:
+                    additionalList = v['__attribute']['additionalList']
+                
             if type == 'argument':
-                tmpRoot = self.addArgument(root,k,argumentType,returnable,desc)
+                tmpRoot = self.addArgument(root,k,argumentType,returnable,desc,prefunc=prefunc,returnfunc=returnfunc,additionalDict=additionalDict,additionalList=additionalList)
             else:
-                tmpRoot = self.addCmd(root,k,'command',returnable,desc)
-            if ('__attribute' in v and len(v) > 1) or ('__attribute' not in v and len(v) > 0):
+                tmpRoot = self.addCmd(root,k,'command',returnable,desc,prefunc=prefunc,returnfunc=returnfunc,additionalDict=additionalDict,additionalList=additionalList)
+            if v and (('__attribute' in v and len(v) > 1) or ('__attribute' not in v and len(v) > 0)):
                 self._setCliRuleTcmdRecursive(tmpRoot,v)
             
     def setCliRuleTcmd(self,top):
@@ -529,8 +542,7 @@ class CiscoStyleCli:
                     _to['__additionalDict__'] = {}
                 _to['__additionalDict__'][adk] = adv
         if 'additionalList' in _from:
-            for adk,adv in _from['additionalList']:
-                _to['__additionalList__'] = _from['additionalList']
+            _to['__additionalList__'] = _from['additionalList']
     def _changeQuoteFlag(self,quoteFlag,s = None):
         if s == None:
             if quoteFlag == False:
@@ -639,7 +651,7 @@ class CiscoStyleCli:
                     cmdRoot[s]['prefunc'](retValue)
                 if self.debug:
                     print('recommend: ({})'.format(t) , s , "-" , cmdRoot[s]['desc'] ,returnable , flush=True)
-                print('help: ({})'.format(t) , s , "-" , cmdRoot[s]['desc'] ,returnable , flush=True)
+                print('    -> ({})'.format(t) , s , "-" , cmdRoot[s]['desc'] ,returnable , flush=True)
                 if cmdRoot[s]['type'] == 'argument' and 'argument-type' in cmdRoot[s] :
                     ld = cmdRoot[s]['argument-type']
                     if isinstance(ld,list) :
@@ -699,7 +711,7 @@ class CiscoStyleCli:
         quoteFlag = False
         isFinishedFromReturn = False
         if self.debug:
-            print("cmd:[",self.cmd,"]",sep="")
+            print("cmd:[",cmd,"]",sep="")
             print("checkCmd:words:",words)
 
         root = self.remoteCmd
@@ -784,12 +796,13 @@ class CiscoStyleCli:
                     retValue['__cmd__'] = retCmdList
                     retValue['__return__'] = newCmd.strip().replace('\t',' ')
                     self._copyAdditionalDictAndList(root,retValue)
+                    if self.debug:
+                        print('matched command : check returnable : root:',root)
                     if 'returnable' in root and root['returnable'] == 'returnable':
                         if 'returnfunc' in root and root['returnfunc']:
                             if self.debug:
-                                print('returnfunc:',root['returnfunc'])
-                                print("retValue:",retValue)
-                            print('returnfunc',root['returnfunc'])
+                                print('matched command returnfunc:',root['returnfunc'])
+                                print("matched command retValue:",retValue)
                             root['returnfunc'](retValue)
                         self.cmd = ""
                         if self.debug:
